@@ -6,7 +6,7 @@
  *
  * Copyright (C) 2002-2004
  *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -64,31 +64,38 @@ bool gcpArrow::Save(xmlDocPtr xml, xmlNodePtr node)
 	return true;
 }
 
-bool gcpArrow::Load(xmlNodePtr node)
+bool gcpArrow::Load (xmlNodePtr node)
 {
 	char* tmp, *endptr;
 	xmlNodePtr child;
-	tmp = (char*)xmlGetProp(node, (xmlChar*)"id");
-	if (tmp) SetId(tmp);
-	child = GetNodeByName(node, "start");
+	tmp = (char*) xmlGetProp (node, (xmlChar*) "id");
+	if (tmp) {
+		SetId (tmp);
+		xmlFree (tmp);
+	}
+	child = GetNodeByName (node, "start");
 	if (!child) return false;
-	tmp = (char*)xmlGetProp(child, (xmlChar*)"x");
+	tmp = (char*) xmlGetProp (child, (xmlChar*) "x");
 	if (!tmp) return false;
-	m_x = strtod(tmp, &endptr);
+	m_x = strtod (tmp, &endptr);
+	xmlFree (tmp);
 	if (*endptr) return false;
-	tmp = (char*)xmlGetProp(child, (xmlChar*)"y");
+	tmp = (char*) xmlGetProp (child, (xmlChar*) "y");
 	if (!tmp) return false;
-	m_y = strtod(tmp, &endptr);
+	m_y = strtod (tmp, &endptr);
+	xmlFree (tmp);
 	if (*endptr) return false;
-	child = GetNodeByName(node, "end");
+	child = GetNodeByName (node, "end");
 	if (!child) return false;
-	tmp = (char*)xmlGetProp(child, (xmlChar*)"x");
+	tmp = (char*) xmlGetProp (child, (xmlChar*) "x");
 	if (!tmp) return false;
-	m_width = strtod(tmp, &endptr) - m_x;
+	m_width = strtod (tmp, &endptr) - m_x;
+	xmlFree (tmp);
 	if (*endptr) return false;
-	tmp = (char*)xmlGetProp(child, (xmlChar*)"y");
+	tmp = (char*) xmlGetProp (child, (xmlChar*) "y");
 	if (!tmp) return false;
-	m_height = strtod(tmp, &endptr) - m_y;
+	m_height = strtod (tmp, &endptr) - m_y;
+	xmlFree (tmp);
 	if (*endptr) return false;
 	return true;
 }
