@@ -43,16 +43,11 @@ gcpFileChooser::gcpFileChooser(gcpApplication* App, bool Save, gcpDocument* pDoc
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 	GtkFileChooser* chooser = GTK_FILE_CHOOSER (dialog);
 	GtkFileFilter* filter = gtk_file_filter_new ();
-	gtk_file_filter_add_pattern (filter, "*.gchempaint");
-	gtk_file_filter_add_pattern (filter, "*.cml");
-	gtk_file_filter_add_pattern (filter, "*.mol");
-	gtk_file_filter_add_pattern (filter, "*.pdb");
 	gtk_file_filter_add_mime_type (filter, "application/x-gchempaint");
 	gtk_file_filter_add_mime_type (filter, "chemical/x-chemdraw");
 	gtk_file_filter_add_mime_type (filter, "chemical/x-cml");
 	gtk_file_filter_add_mime_type (filter, "chemical/x-mdl-molfile");
 	gtk_file_filter_add_mime_type (filter, "chemical/x-pdb");
-	gtk_file_filter_add_mime_type (filter, "chemical/x-pdb");
 	gtk_file_chooser_set_filter (chooser, filter);
 	if (!Save) gtk_file_chooser_set_select_multiple (chooser, true);
 	while (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
