@@ -6,7 +6,7 @@
  *
  * Copyright (C) 2002-2004
  *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -36,6 +36,7 @@
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include "libgcpcanvas/gcp-canvas-rect-ellipse.h"
 #include "libgcpcanvas/gcp-canvas-bpath.h"
+#include "libgcpcanvas/gcp-canvas-text.h"
 #include <gcu/element.h>
 #include <math.h>
 
@@ -88,7 +89,7 @@ bool gcpFragment::OnChanged(GtkTextBuffer *textbuffer)
 		m_bLoading = false;
 		return false;
 	}
-	GnomeCanvasRichTextExt* text = (GnomeCanvasRichTextExt*)g_object_get_data(G_OBJECT(pData->Items[this]), "fragment");
+	GnomeCanvasRichTextExt* text = (GnomeCanvasRichTextExt*) g_object_get_data(G_OBJECT(pData->Items[this]), "fragment");
 	PangoLayout* layout = gnome_canvas_rich_text_ext_get_pango_layout(text, 0);
 	if (layout)
 	{
@@ -99,6 +100,15 @@ bool gcpFragment::OnChanged(GtkTextBuffer *textbuffer)
 	}
 	GdkRectangle rect;
 	GtkTextIter start, iter;
+	if (!gtk_text_buffer_get_char_count (m_buf)) {
+		gtk_text_buffer_get_start_iter(m_buf,&iter);
+		gnome_canvas_rich_text_ext_get_iter_location(text, &iter, &rect);
+		m_height = rect.height + 1;
+		m_length = 1;
+		pView->Update (this);
+		m_bLoading = false;
+		return true;
+	}
 	gtk_text_buffer_get_end_iter(m_buf,&iter);
 	gnome_canvas_rich_text_ext_get_iter_location(text, &iter, &rect);
 	m_length = rect.x + 1;
@@ -194,6 +204,7 @@ bool gcpFragment::OnChanged(GtkTextBuffer *textbuffer)
 		pApp->ActivateMenu("SaveAs", false);
 		pApp->ActivateTool("Save", false);
 	}
+	EmitSignal (OnChangedSignal);
 	return true;
 }
 
@@ -335,93 +346,62 @@ void gcpFragment::Add(GtkWidget* w)
 	gnome_canvas_rich_text_ext_set_buffer(GNOME_CANVAS_RICH_TEXT_EXT(item), m_buf);
 	g_object_set_data(G_OBJECT(item), "object", this);
 	g_signal_connect(G_OBJECT(item), "event", G_CALLBACK(on_event), w);
-	AddChargeItem(group);
-	pData->Items[this] = group;
-	if (g_object_get_data(G_OBJECT(item), "realized")) OnChanged(m_buf);
-}
-
-void gcpFragment::AddChargeItem(GnomeCanvasGroup* group)
-{
-	gcpDocument* pDoc = (gcpDocument*)GetDocument();
-	if (!pDoc) return;
-	GtkWidget* pWidget = pDoc->GetView()->GetWidget();
-	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(pWidget), "data");
-	double x, y;
-	GnomeCanvasItem* item;
-	GnomeCanvasPathDef* path = gnome_canvas_path_def_new ();
-	if (!m_Atom->GetCharge()) {
-		x = m_x * pData->ZoomFactor;
-		y = m_y * pData->ZoomFactor;
-		item = gnome_canvas_item_new(
-						group,
-						gnome_canvas_ellipse_ext_get_type(),
-						"width_units", 0.,
-						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
-						"x1", x - 1.,
-						"x2", x + 1.,
-						"y1",  y - 1.,
-						"y2", y + 1.,
-						NULL);
-		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
-		g_object_set_data ((GObject*)group, "circle", item);
-		gnome_canvas_path_def_moveto (path, x - 0.01, y);
-		gnome_canvas_path_def_lineto (path, x + 0.01, y);
-		item = gnome_canvas_item_new(
-						group,
-						gnome_canvas_bpath_ext_get_type(),
-						"bpath", path,
-						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
-						"width_units", 0.01,
-						NULL);
-		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
-		g_object_set_data ((GObject*)group, "charge", item);
-	} else {
-		int align = GetChargePosition(m_Atom, x, y);
+	/* add charge */
+	int charge = !m_Atom->GetCharge ();
+	if (charge) {
+		double x, y, Angle, Dist;
+		char Pos = m_Atom->gcpAtom::GetChargePosition (&Angle, &Dist);
+		int align = GetChargePosition(m_Atom, Pos, 0., x, y);
+		if (Dist != 0.) {
+			x = m_x + Dist * cos (Angle);
+			y = m_y - Dist * sin (Angle);
+		}
 		x *= pData->ZoomFactor;
 		y *= pData->ZoomFactor;
+		GtkAnchorType anchor;
 		switch (align)
 		{
 			case -2:
-				y += 6.0;//FIXME: change 6.0 end other constants to variables
+				y += 6.0;
+				anchor = GTK_ANCHOR_NORTH;
 				break;
 			case -1:
 				x -= 6.0;
+				anchor = GTK_ANCHOR_EAST;
+				break;
+			case 0:
+				anchor = GTK_ANCHOR_CENTER;
 				break;
 			case 1:
 				x += 6.0;
+				anchor = GTK_ANCHOR_WEST;
 				break;
 			case 2:
 				y -= 6.0;
+				anchor = GTK_ANCHOR_SOUTH;
 				break;
 		}
+		char *markup;
+		if 	(abs (charge) > 1)
+			markup = g_strdup_printf ("%d<span size=\"larger\">%s</span>", abs (charge), (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
+		else
+			markup = g_strdup_printf ("<span size=\"larger\">%s</span>", (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
 		item = gnome_canvas_item_new(
 						group,
-						gnome_canvas_ellipse_ext_get_type(),
-						"width_units", 1.0,
-						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
-						"x1", x - 4.0,
-						"x2", x + 4.0,
-						"y1", y - 4.0,
-						"y2", y + 4.0,
-						NULL);
-		g_object_set_data ((GObject*)group, "circle", item);
-		gnome_canvas_path_def_moveto (path, x - 2.5, y);
-		gnome_canvas_path_def_lineto (path, x + 2.5, y);
-		if (m_Atom->GetCharge() > 0)
-		{	
-			gnome_canvas_path_def_moveto (path, x, y - 2.5);
-			gnome_canvas_path_def_lineto (path, x, y + 2.5);
-		}
-		item = gnome_canvas_item_new(
-						group,
-						gnome_canvas_bpath_ext_get_type(),
-						"bpath", path,
-						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
-						"width_units", 1.0,
+						gnome_canvas_text_ext_get_type(),
+						"fill_color", (pData->IsSelected(this))? SelectColor: Color,
+						"font-desc", pData->View->GetPangoSmallFontDesc (),
+						"markup", markup,
+						"anchor", anchor,
+						"x", x,
+						"y", y,
 						NULL);
+		g_free (markup);
 		g_object_set_data ((GObject*)group, "charge", item);
 	}
-	gnome_canvas_path_def_unref (path);
+	pData->Items[this] = group;
+	gpointer obj = g_object_get_data (G_OBJECT (group), "fragment");
+	if (g_object_get_data(G_OBJECT (obj), "realized")) OnChanged(m_buf);
 }
 
 void gcpFragment::SetSelected(GtkWidget* w, int state)
@@ -438,10 +418,9 @@ void gcpFragment::SetSelected(GtkWidget* w, int state)
 	}
 	g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
 				"fill_color", color, NULL);
-	g_object_set (G_OBJECT(g_object_get_data(G_OBJECT(group), "circle")),
-				"outline_color", chargecolor, NULL);
-	g_object_set (G_OBJECT(g_object_get_data(G_OBJECT(group), "charge")),
-				"outline_color", chargecolor, NULL);
+	gpointer item = g_object_get_data (G_OBJECT (group), "charge");
+	if (item)
+		g_object_set(item, "fill_color", chargecolor, NULL);
 }
 
 void gcpFragment::Update(GtkWidget* w)
@@ -460,72 +439,101 @@ void gcpFragment::Update(GtkWidget* w)
 						"x2", m_x * pData->ZoomFactor + m_length + pData->Padding - m_lbearing,
 						"y2", m_y * pData->ZoomFactor + m_height + pData->Padding - m_ascent + m_CHeight,
 						NULL);
-	GnomeCanvasPathDef* path = gnome_canvas_path_def_new ();
-	void* item;
-	double x, y;
-	if (!m_Atom->GetCharge()) {
-		x = m_x * pData->ZoomFactor;
-		y = m_y * pData->ZoomFactor;
-		item = g_object_get_data(G_OBJECT(group), "circle");
-		g_object_set (item,
-						"width_units", 0.,
-						"x1", x - 1,
-						"x2", x + 1,
-						"y1", y - 1,
-						"y2", y + 1,
+	void* item = g_object_get_data(G_OBJECT(group), "charge");
+	int charge = m_Atom->GetCharge ();
+	if (charge) {
+		double x, y, Angle, Dist;
+		char Pos = m_Atom->gcpAtom::GetChargePosition (&Angle, &Dist);
+		if (item) {
+			int align = GetChargePosition(m_Atom, Pos, Angle, x, y);
+			if (Dist != 0.) {
+				x = m_x + Dist * cos (Angle);
+				y = m_y - Dist * sin (Angle);
+			}
+			x *= pData->ZoomFactor;
+			y *= pData->ZoomFactor;
+			GtkAnchorType anchor;
+			switch (align)
+			{
+				case -2:
+					anchor = GTK_ANCHOR_NORTH;
+					break;
+				case -1:
+					anchor = GTK_ANCHOR_EAST;
+					break;
+				case 0:
+					anchor = GTK_ANCHOR_CENTER;
+					break;
+				case 1:
+					anchor = GTK_ANCHOR_WEST;
+					break;
+				case 2:
+					anchor = GTK_ANCHOR_SOUTH;
+					break;
+			}
+			char *markup;
+			if 	(abs (charge) > 1)
+				markup = g_strdup_printf ("%d<span size=\"larger\">%s</span>", abs (charge), (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
+			else
+				markup = g_strdup_printf ("<span size=\"larger\">%s</span>", (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
+			g_object_set (G_OBJECT (item),
+						"markup", markup,
+						"anchor", anchor,
+						"x", x,
+						"y", y,
 						NULL);
-		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
-		gnome_canvas_path_def_moveto (path, x - 0.01, y);
-		gnome_canvas_path_def_lineto (path, x + 0.01, y);
-		item = g_object_get_data(G_OBJECT(group), "charge");
-		g_object_set (item,
-						"bpath", path,
-						"width_units", 0.,
-						NULL);
-		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
+			g_free (markup);
+		} else {
+			int align = GetChargePosition(m_Atom, Pos, Angle, x, y);
+			x *= pData->ZoomFactor;
+			if (Dist != 0.) {
+				x = m_x + Dist * cos (Angle);
+				y = m_y - Dist * sin (Angle);
+			}
+			y *= pData->ZoomFactor;
+			GtkAnchorType anchor;
+			switch (align)
+			{
+				case -2:
+					anchor = GTK_ANCHOR_NORTH;
+					break;
+				case -1:
+					anchor = GTK_ANCHOR_EAST;
+					break;
+				case 0:
+					anchor = GTK_ANCHOR_CENTER;
+					break;
+				case 1:
+					anchor = GTK_ANCHOR_WEST;
+					break;
+				case 2:
+					anchor = GTK_ANCHOR_SOUTH;
+					break;
+			}
+			char *markup;
+			if 	(abs (charge) > 1)
+				markup = g_strdup_printf ("%d<span size=\"larger\">%s</span>", abs (charge), (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
+			else
+				markup = g_strdup_printf ("<span size=\"larger\">%s</span>", (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
+			item = gnome_canvas_item_new(
+							group,
+							gnome_canvas_text_ext_get_type(),
+							"fill_color", (pData->IsSelected(this))? SelectColor: Color,
+							"font-desc", pData->View->GetPangoSmallFontDesc (),
+							"markup", markup,
+							"anchor", anchor,
+							"x", x,
+							"y", y,
+							NULL);
+			g_free (markup);
+			g_object_set_data ((GObject*)group, "charge", item);
+		}
+		gnome_canvas_item_show (GNOME_CANVAS_ITEM (item));
 	} else {
-		int align = GetChargePosition(m_Atom, x, y);
-		x *= pData->ZoomFactor;
-		y *= pData->ZoomFactor;
-		switch (align)
-		{
-			case -2:
-				y += 6.0;//FIXME: change 6.0 end other constants to variables
-				break;
-			case -1:
-				x -= 6.0;
-				break;
-			case 1:
-				x += 6.0;
-				break;
-			case 2:
-				y -= 6.0;
-				break;
-		}
-		item = g_object_get_data(G_OBJECT(group), "circle");
-		g_object_set (item,
-						"width_units", 1.0,
-						"x1", x - 4.0,
-						"x2", x + 4.0,
-						"y1", y - 4.0,
-						"y2", y + 4.0,
-						NULL);
-		gnome_canvas_item_show (GNOME_CANVAS_ITEM (item));
-		gnome_canvas_path_def_moveto (path, x - 2.5, y);
-		gnome_canvas_path_def_lineto (path, x + 2.5, y);
-		if (m_Atom->GetCharge() > 0)
-		{	
-			gnome_canvas_path_def_moveto (path, x, y - 2.5);
-			gnome_canvas_path_def_lineto (path, x, y + 2.5);
-		}
-		item = g_object_get_data(G_OBJECT(group), "charge");
-		g_object_set (item,
-						"bpath", path,
-						"width_units", 1.0,
-						NULL);
-		gnome_canvas_item_show (GNOME_CANVAS_ITEM (item));
+		if (item)
+			gtk_object_destroy (GTK_OBJECT (item));
+			g_object_set_data ((GObject*)group, "charge", NULL);
 	}
-	gnome_canvas_path_def_unref (path);
 }
 
 xmlNodePtr gcpFragment::Save(xmlDocPtr xml)
@@ -706,8 +714,9 @@ bool gcpFragment::Load(xmlNodePtr node)
 	{
 		if (!strcmp((const char*)child->name, "text"))
 		{
-			tmp = (char*) xmlNodeGetContent(child);
-			gtk_text_buffer_insert(m_buf, &iter, tmp, -1);
+			tmp = (char*) xmlNodeGetContent (child);
+			gtk_text_buffer_insert (m_buf, &iter, tmp, -1);
+			xmlFree (tmp);
 		}
 		else if (!strcmp((const char*)child->name, "atom"))
 		{
@@ -724,8 +733,9 @@ bool gcpFragment::Load(xmlNodePtr node)
 		}
 		else if (!strcmp((const char*)child->name, "charge"))
 		{
-			tmp = (char*) xmlGetProp(child, (xmlChar*)"value");
-			int charge = atoi(tmp);
+			tmp = (char*) xmlGetProp (child, (xmlChar*) "value");
+			int charge = atoi (tmp);
+			xmlFree (tmp);
 			if (abs(charge) > 1)
 				tmp = g_strdup_printf("%d%c",charge, (charge > 0)? '+': '-');
 			else if (charge == 1)
@@ -999,7 +1009,7 @@ int gcpFragment::GetElementAtIter(GtkTextIter* start, GtkTextIter* end)
 	return 0;
 }
 
-int gcpFragment::GetChargePosition(gcpFragmentAtom* pAtom, double& x, double& y)
+int gcpFragment::GetChargePosition(gcpFragmentAtom* pAtom, char& Pos, double Angle, double& x, double& y)
 {
 	if ((pAtom != m_Atom) || (m_Atom->GetZ() == 0)) return 0;
 	GtkTextIter iter;
@@ -1043,58 +1053,66 @@ int gcpFragment::GetChargePosition(gcpFragmentAtom* pAtom, double& x, double& y)
 		if ((result & CHARGE_E) && ((angle <= 225.0) && (angle >= 135.0))) result -= CHARGE_E;
 		if ((result & CHARGE_W) && (angle >= 315.0) || (angle <= 45.0)) result -= CHARGE_W;
 	}
-	if (result)
-	{
-		if (result & CHARGE_NE)
-		{
-			x = m_x + width / 2.0;
-			y = m_y - height / 2.0;
-			return 1;
-		}
-		if (result & CHARGE_NW)
-		{
-			x = m_x - width / 2.0;
-			y = m_y - height / 2.0;
-			return -1;
-		}
-		if (result & CHARGE_N)
-		{
-			x = m_x;
-			y = m_y - height / 2.0;
-			return 2;
-		}
-		if (result & CHARGE_SE)
-		{
-			x = m_x + width / 2.0;
-			y = m_y + height / 2.0;
-			return 1;
-		}
-		if (result & CHARGE_SW)
-		{
-			x = m_x - width / 2.0;
-			y = m_y + height / 2.0;
-			return -1;
-		}
-		if (result & CHARGE_S)
-		{
-			x = m_x;
-			y = m_y + height / 2.0;
-			return -2;
-		}
-		if (result & CHARGE_E)
-		{
-			x = m_x + width / 2.0;
-			y = m_y;
-			return 1;
-		}
-		if (result & CHARGE_W)
-		{
-			x = m_x - width / 2.0;
-			y = m_y;
-			return -1;
-		}
+	if (Pos == -1) {
+		if (result) {
+			if (result & CHARGE_NE)
+				Pos = CHARGE_NE;
+			else if (result & CHARGE_NW)
+				Pos = CHARGE_NW;
+			else if (result & CHARGE_N)
+				Pos = CHARGE_N;
+			else if (result & CHARGE_SE)
+				Pos = CHARGE_SE;
+			else if (result & CHARGE_SW)
+				Pos = CHARGE_SW;
+			else if (result & CHARGE_S)
+				Pos = CHARGE_S;
+			else if (result & CHARGE_E)
+				Pos = CHARGE_E;
+			else if (result & CHARGE_W)
+				Pos = CHARGE_W;
+		} else
+			return 0;
+	} else if (Pos) {
+		if (!(Pos & result))
+			return 0;
+	} else
+		return 0;
+	switch (Pos) {
+	case CHARGE_NE:
+		x = m_x + width / 2.0;
+		y = m_y - height / 2.0;
+		return 1;
+	case  CHARGE_NW:
+		x = m_x - width / 2.0;
+		y = m_y - height / 2.0;
+		return -1;
+	case  CHARGE_N:
+		x = m_x;
+		y = m_y - height / 2.0;
+		return 2;
+	case CHARGE_SE:
+		x = m_x + width / 2.0;
+		y = m_y + height / 2.0;
+		return 1;
+	case CHARGE_SW:
+		x = m_x - width / 2.0;
+		y = m_y + height / 2.0;
+		return -1;
+	case CHARGE_S:
+		x = m_x;
+		y = m_y + height / 2.0;
+		return -2;
+	case CHARGE_E:
+		x = m_x + width / 2.0;
+		y = m_y;
+		return 1;
+	case CHARGE_W:
+		x = m_x - width / 2.0;
+		y = m_y;
+		return -1;
 	}
-	return result;
+	return 0;
 }
 
 bool gcpFragment::Validate()
@@ -1168,3 +1186,13 @@ double gcpFragment::GetYAlign ()
 {
 	return m_y;
 }
+
+int gcpFragment::GetAvailablePosition(double& x, double& y)
+{
+	return 0;
+}
+
+bool gcpFragment::GetPosition(double angle, double& x, double& y)
+{
+	return false;
+}
