@@ -50,6 +50,9 @@ public:
 	virtual void Transform2D(Matrix2D& m, double x, double y);
 	virtual Object* GetAtomAt(double x, double y, double z = 0.);
 	virtual double GetYAlign ();
+	virtual bool BuildContextualMenu (GtkUIManager *UIManager);
+	void ExportToGhemical ();
+	void SelectAlignmentItem ();
 
 private:
 	list<gcpCycle*> m_Cycles;
@@ -57,6 +60,7 @@ private:
 	list<gcpAtom*> m_Atoms;
 	list<gcpFragment*> m_Fragments;
 	list<gcpBond*> m_Bonds;
+	Object *m_Alignment;
 };
 
 #endif // GCHEMPAINT_MOLECULE_H
