@@ -35,8 +35,10 @@
 #include "libgcpcanvas/gprintable.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include <math.h>
+#include <unistd.h>
 #include <pango/pango-context.h>
 #include <gdk/gdkkeysyms.h>
+#include <libgnomeprint/gnome-print-job.h>
 
 /*
 Derivation of a new widget from gnome_canvas with an event for updating canvas size
@@ -205,6 +207,8 @@ gcpView::gcpView(gcpDocument *pDoc, bool Embedded)
 	m_height = 300;
 	m_ActiveRichText = NULL;
 	if (!(m_bEmbedded = Embedded)) CreateNewWidget();
+	m_UIManager = gtk_ui_manager_new ();
+	m_Dragging = false;
 }
 
 gcpView::~gcpView()
@@ -212,10 +216,9 @@ gcpView::~gcpView()
 	g_object_unref(G_OBJECT(m_PangoContext));
 	if (m_sFontName) g_free(m_sFontName);
 	if (m_sSmallFontName) g_free(m_sSmallFontName);
+	g_object_unref (m_UIManager);
 }
 
-static bool bDragging = false;
-
 bool gcpView::OnEvent(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 {
 	gcpTool* pActiveTool = m_pDoc->GetApplication()->GetActiveTool();
@@ -263,14 +266,14 @@ bool gcpView::OnEvent(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 		{
 			case 1:
 			{
-				if (bDragging) break;
+				if (m_Dragging) break;
 				bool result = pActiveTool->OnClicked(this, Obj, x, y, event->button.state);
 				if (item && (item == (GnomeCanvasItem*)m_ActiveRichText))
 				{
 					GnomeCanvasItemClass* klass = GNOME_CANVAS_ITEM_CLASS(((GTypeInstance*)item)->g_class);
 					return klass->event(item, event);
 				}
-				bDragging = result;
+				m_Dragging = result;
 				return true;
 			}
 			case 2:
@@ -281,23 +284,31 @@ bool gcpView::OnEvent(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 				OnPasteSelection (m_pWidget, clipboard);
 				return true;
 			}
-			case 3:
-				if (pActiveTool->OnRightButtonClicked(this, Obj, event->button.x, event->button.y))
-					return true;
+			case 3: {
+				bool result;
+				g_object_unref (m_UIManager);
+			 	m_UIManager = gtk_ui_manager_new ();
+				result = pActiveTool->OnRightButtonClicked(this, Obj, event->button.x, event->button.y, m_UIManager);
 				if (Obj)
-					Obj->ShowContextualMenu(event->button.button, event->button.time);
+					result |= Obj->BuildContextualMenu (m_UIManager);
+				if (result) {
+					GtkWidget *w = gtk_ui_manager_get_widget (m_UIManager, "/popup");
+					gtk_menu_popup (GTK_MENU (w), NULL, NULL, NULL, NULL, 3,  gtk_get_current_event_time ());
+					return true;
+				}
+			}
 		}
 		break;
 	case GDK_MOTION_NOTIFY:
-		if (!bDragging) break;
+		if (!m_Dragging) break;
 		pActiveTool->OnDrag(x, y, event->button.state);
 		return true;
 	case GDK_BUTTON_RELEASE:
 	switch (event->button.button)
 		{
 		case 1:
-			if (!bDragging) break;
-			bDragging = false;
+			if (!m_Dragging) break;
+			m_Dragging = false;
 			pActiveTool->OnRelease(x, y, event->button.state);
 			m_pDoc->GetApplication()->ClearStatus();
 			return true;
@@ -735,7 +746,7 @@ void gcpView::UpdateSize(double x1, double y1, double x2, double y2)
 void gcpView::SetGnomeCanvasRichTextActive(GnomeCanvasRichTextExt* item)
 {
 	m_ActiveRichText = item;
-	bDragging = false;
+	m_Dragging = false;
 }
 
 bool gcpView::PrepareUnselect()
@@ -758,3 +769,104 @@ void gcpView::OnSelectAll()
 			(m_pData->SelectedObjects.back()->GetType() == MoleculeType)));
 	}
 }
+
+void gcpView::ExportImage (const char* filename, const char* type)
+{
+	ArtDRect rect;
+	m_pData->GetObjectBounds (m_pDoc, &rect);
+	m_pData->ShowSelection (false);
+	int w = (int) (ceil (rect.x1) - floor (rect.x0)), h = (int) (ceil (rect.y1) - floor (rect.y0));
+	if (!strcmp (type, "eps")) {
+		GnomePrintConfig* config = gnome_print_config_default();
+		GnomePrintContext *pc;
+		GnomePrintJob *gpj = gnome_print_job_new (config);
+		pc = gnome_print_job_get_context (gpj);
+		gnome_print_beginpage(pc, (const guchar*)"");
+		gdouble width, height;
+		gnome_print_config_get_page_size(config, &width, &height);
+		Print(pc, width, height);
+		gnome_print_showpage(pc);
+		g_object_unref(pc);
+		gnome_print_job_close(gpj);
+		char *tmpname = g_strdup ("/tmp/gcp2epsXXXXXX");
+		int f = g_mkstemp (tmpname);
+		close (f);
+		gnome_print_config_set_boolean (config, (const guchar*) "Settings.Output.Job.PrintToFile", true);
+		gnome_print_config_set (config, (const guchar*) GNOME_PRINT_KEY_OUTPUT_FILENAME, (const guchar*) tmpname);
+		gnome_print_job_print (gpj);
+		g_object_unref (gpj);
+		gnome_print_config_unref(config);
+		char *command_line = g_strconcat ("ps2eps ", tmpname, NULL);
+		g_spawn_command_line_sync (command_line, NULL, NULL, NULL, NULL);
+		g_free (command_line);
+		command_line = g_strconcat ("mv -f ", tmpname, ".eps ", filename);
+		g_spawn_command_line_async (command_line, NULL);
+		remove (tmpname);
+		g_free (tmpname);
+		g_free (command_line);
+	} else if (!strcmp (type, "svg")) {
+		xmlDocPtr doc = xmlNewDoc ((const xmlChar*)"1.0");
+		char *old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
+		char *buf;
+		setlocale (LC_NUMERIC, "C");
+		xmlNewDtd (doc,
+			   (const xmlChar*)"svg", (const xmlChar*)"-//W3C//DTD SVG 1.1//EN",
+			   (const xmlChar*)"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd");
+		doc->children = xmlNewDocNode (doc, NULL, (const xmlChar*)"svg", NULL);
+		xmlNsPtr ns = xmlNewNs (doc->children, (const xmlChar*)"http://www.w3.org/2000/svg", NULL);
+		xmlSetNs (doc->children, ns);
+		xmlNewProp (doc->children, (const xmlChar*)"version", (const xmlChar*)"1.1");
+		rect.x0 = floor (rect.x0);
+		rect.y0 = floor (rect.y0);
+		rect.x1 = ceil (rect.x1);
+		rect.y1 = ceil (rect.y1);
+		buf = g_strdup_printf ("%g", rect.x1 - rect.x0);
+		xmlNewProp (doc->children, (const xmlChar*)"width", (const xmlChar*)buf);
+		g_free (buf);
+		buf = g_strdup_printf ("%g", rect.y1 - rect.y0);
+		xmlNewProp (doc->children, (const xmlChar*)"height", (const xmlChar*)buf);
+		g_free (buf);
+		xmlNodePtr node;
+		node = xmlNewDocNode (doc, NULL, (const xmlChar*)"rect", NULL);
+		xmlAddChild (doc->children, node);
+		buf = g_strdup_printf ("%g", rect.x1 - rect.x0);
+		xmlNewProp (node, (const xmlChar*)"width", (const xmlChar*)buf);
+		g_free (buf);
+		buf = g_strdup_printf ("%g", rect.y1 - rect.y0);
+		xmlNewProp (node, (const xmlChar*)"height", (const xmlChar*)buf);
+		g_free (buf);
+		xmlNewProp (node, (const xmlChar*)"stroke", (const xmlChar*)"none");
+		xmlNewProp (node, (const xmlChar*)"fill", (const xmlChar*)"white");
+		if (rect.x0 != 0. || rect.y0 != 0) {
+			node = xmlNewDocNode (doc, NULL, (const xmlChar*)"g", NULL);
+			xmlAddChild (doc->children, node);
+			buf = g_strdup_printf ("translate(%g,%g)", - rect.x0, - rect.y0);
+			xmlNewProp (node, (const xmlChar*)"transform", (const xmlChar*)buf);
+			g_free (buf);
+		} else
+			node = doc->children;
+		g_printable_export_svg (G_PRINTABLE (m_pData->Group), doc, node);
+		setlocale (LC_NUMERIC, old_num_locale);
+		g_free (old_num_locale);
+		xmlIndentTreeOutput = true;
+		xmlKeepBlanksDefault (0);
+		xmlSaveFormatFile (filename, doc, true);
+		xmlFreeDoc (doc);
+	} else {
+		gnome_canvas_update_now (GNOME_CANVAS (m_pWidget));
+		GdkDrawable *drawable = GDK_DRAWABLE (gtk_widget_get_parent_window (m_pWidget));
+		GdkPixbuf *pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, w, h);
+		GnomeCanvasBuf buf;
+		buf.buf = gdk_pixbuf_get_pixels (pixbuf);
+		buf.rect.x0 = (int) floor (rect.x0);
+		buf.rect.x1 = (int) ceil (rect.x1);
+		buf.rect.y0 = (int) floor (rect.y0);
+		buf.rect.y1 = (int) ceil (rect.y1);
+		buf.buf_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+		buf.bg_color = 0xffffff;
+		(* GNOME_CANVAS_ITEM_GET_CLASS (m_pData->Group)->render) (GNOME_CANVAS_ITEM (m_pData->Group), &buf);
+		gdk_pixbuf_save (pixbuf, filename, type, NULL, NULL);
+		g_object_unref (pixbuf);
+	}
+	m_pData->ShowSelection (true);
+}
