@@ -95,6 +95,7 @@ static double gnome_canvas_line_ext_point       (GnomeCanvasItem *item, double x
 static void   gnome_canvas_line_ext_bounds      (GnomeCanvasItem *item, double *x1, double *y1, double *x2, double *y2);
 static void   gnome_canvas_line_ext_render      (GnomeCanvasItem *item, GnomeCanvasBuf *buf);
 static void   gnome_canvas_line_ext_print       (GPrintable *gprintable, GnomePrintContext *pc);
+static void   gnome_canvas_line_ext_export_svg (GPrintable *printable, xmlDocPtr doc, xmlNodePtr node);
 
 static GnomeCanvasItemClass *parent_class;
 
@@ -102,6 +103,7 @@ static void
 gnome_canvas_line_print_init (GPrintableIface *iface)
 {
 	iface->print = gnome_canvas_line_ext_print;
+	iface->export_svg = gnome_canvas_line_ext_export_svg;
 }
 
 GType
@@ -1152,3 +1154,131 @@ gnome_canvas_line_ext_print (GPrintable *gprintable, GnomePrintContext *pc)
 		gnome_print_fill (pc);
 	}
 }
+
+static void
+gnome_canvas_line_ext_export_svg (GPrintable *printable, xmlDocPtr doc, xmlNodePtr node)
+{
+	GnomeCanvasLine *line;
+	GnomeCanvasLineExt *lineext;
+	gdouble dashes[2] = {3.0, 2.0};
+	GString *string;
+	xmlNodePtr child;
+	char *buf;
+	double width;
+	int opacity, i;
+
+	line = GNOME_CANVAS_LINE (printable);
+	lineext = GNOME_CANVAS_LINE_EXT (printable);
+	
+	if (line->num_points == 0)
+		return;
+
+	child = xmlNewDocNode (doc, NULL, (const xmlChar*) "path", NULL);
+	xmlAddChild (node, child);
+	string = g_string_new ("");
+	g_string_append_printf (string, "M%g %g", line->coords[0], line->coords[1]);
+	for (i = 1; i < line->num_points; i++)
+		g_string_append_printf (string, "L%g %g", line->coords[2 * i], line->coords[2 * i + 1]);/*FIXME: Change that for spline lines!*/
+	xmlNewProp (child, (const xmlChar*)"d", (const xmlChar*)string->str);
+	g_string_free (string, TRUE);
+	xmlNewProp (child, (const xmlChar*)"fill", (const xmlChar*)"none");
+	buf = g_strdup_printf ("#%06x", ((double)(line->fill_rgba >> 24)) / 255.0,
+								    ((double)((line->fill_rgba >> 16) & 0xff)) / 255.0,
+								    ((double)((line->fill_rgba >> 8) & 0xff)) / 255.0);
+	xmlNewProp (child, (const xmlChar*)"stroke", (const xmlChar*)buf);
+	g_free (buf);
+	opacity = line->fill_rgba & 0xff;
+	if (opacity != 255) {
+		buf = g_strdup_printf ("%g", (double) opacity / 255.);
+		xmlNewProp (child, (const xmlChar*)"stroke-opacity", (const xmlChar*)buf);
+		g_free (buf);
+	}
+	if (line->width_pixels)
+		width = (double) line->width / line->item.canvas->pixels_per_unit;
+	else
+		width = line->width;
+	buf = g_strdup_printf ("%g", width);
+	xmlNewProp (child, (const xmlChar*)"stroke-width", (const xmlChar*)buf);
+	g_free (buf);
+
+	switch (line->cap) {
+	case GDK_CAP_ROUND:
+		xmlNewProp (child, (const xmlChar*)"stroke-linecap", (const xmlChar*)"round");
+		break;
+	case GDK_CAP_PROJECTING:
+		xmlNewProp (child, (const xmlChar*)"stroke-linecap", (const xmlChar*)"square");
+		break;
+	default:
+		xmlNewProp (child, (const xmlChar*)"stroke-linecap", (const xmlChar*)"butt");
+		break;
+	}
+
+	switch (line->join) {
+	case GDK_JOIN_ROUND:
+		xmlNewProp (child, (const xmlChar*)"stroke-join", (const xmlChar*)"round");
+		break;
+	case GDK_JOIN_BEVEL:
+		xmlNewProp (child, (const xmlChar*)"stroke-linejoin", (const xmlChar*)"bevel");
+		break;
+	default:
+		xmlNewProp (child, (const xmlChar*)"stroke-linejoin", (const xmlChar*)"miter");
+		break;
+	}
+
+	if (line->line_style == GDK_LINE_ON_OFF_DASH)
+		xmlNewProp (child, (const xmlChar*)"stroke-dasharray", (const xmlChar*)"3,2");
+
+	if (line->first_arrow && line->first_coords) {
+		child = xmlNewDocNode (doc, NULL, (const xmlChar*) "path", NULL);
+		xmlAddChild (node, child);
+		string = g_string_new ("");
+		g_string_append_printf (string, "M%g %g", line->first_coords[0], line->first_coords[1]);
+		g_string_append_printf (string, "L%g %g", line->first_coords[2], line->first_coords[3]);
+		g_string_append_printf (string, "L%g %g", line->first_coords[4], line->first_coords[5]);
+		g_string_append_printf (string, "L%g %g", line->first_coords[6], line->first_coords[7]);
+		g_string_append_printf (string, "L%g %g", line->first_coords[8], line->first_coords[9]);
+		if (lineext->first_arrow_head_style == ARROW_HEAD_BOTH)
+			g_string_append_printf (string, "L%g %g", line->first_coords[10], line->first_coords[11]);
+		xmlNewProp (child, (const xmlChar*)"d", (const xmlChar*)string->str);
+		g_string_free (string, TRUE);
+		xmlNewProp (child, (const xmlChar*)"stroke", (const xmlChar*)"none");
+		buf = g_strdup_printf ("#%06x", ((double)(line->fill_rgba >> 24)) / 255.0,
+										((double)((line->fill_rgba >> 16) & 0xff)) / 255.0,
+										((double)((line->fill_rgba >> 8) & 0xff)) / 255.0);
+		xmlNewProp (child, (const xmlChar*)"fill", (const xmlChar*)buf);
+		g_free (buf);
+		opacity = line->fill_rgba & 0xff;
+		if (opacity != 255) {
+			buf = g_strdup_printf ("%g", (double) opacity / 255.);
+			xmlNewProp (child, (const xmlChar*)"fill-opacity", (const xmlChar*)buf);
+			g_free (buf);
+		}
+	}
+
+	if (line->last_arrow && line->last_coords) {
+		child = xmlNewDocNode (doc, NULL, (const xmlChar*) "path", NULL);
+		xmlAddChild (node, child);
+		string = g_string_new ("");
+		g_string_append_printf (string, "M%g %g", line->last_coords[0], line->last_coords[1]);
+		g_string_append_printf (string, "L%g %g", line->last_coords[2], line->last_coords[3]);
+		g_string_append_printf (string, "L%g %g", line->last_coords[4], line->last_coords[5]);
+		g_string_append_printf (string, "L%g %g", line->last_coords[6], line->last_coords[7]);
+		g_string_append_printf (string, "L%g %g", line->last_coords[8], line->last_coords[9]);
+		if (lineext->last_arrow_head_style == ARROW_HEAD_BOTH)
+			g_string_append_printf (string, "L%g %g", line->last_coords[10], line->last_coords[11]);
+		xmlNewProp (child, (const xmlChar*)"d", (const xmlChar*)string->str);
+		g_string_free (string, TRUE);
+		xmlNewProp (child, (const xmlChar*)"stroke", (const xmlChar*)"none");
+		buf = g_strdup_printf ("#%06x", ((double)(line->fill_rgba >> 24)) / 255.0,
+										((double)((line->fill_rgba >> 16) & 0xff)) / 255.0,
+										((double)((line->fill_rgba >> 8) & 0xff)) / 255.0);
+		xmlNewProp (child, (const xmlChar*)"fill", (const xmlChar*)buf);
+		g_free (buf);
+		opacity = line->fill_rgba & 0xff;
+		if (opacity != 255) {
+			buf = g_strdup_printf ("%g", (double) opacity / 255.);
+			xmlNewProp (child, (const xmlChar*)"fill-opacity", (const xmlChar*)buf);
+			g_free (buf);
+		}
+	}
+}
