@@ -146,7 +146,7 @@ static void gnome_canvas_rich_text_ext_move_cursor(GnomeCanvasRichTextExt *text,
 					       gint count,
 					       gboolean extend_selection);
 static void   gnome_canvas_rich_text_ext_print       (GPrintable *gprintable, GnomePrintContext *pc);
-
+static void gnome_canvas_rich_text_ext_export_svg   (GPrintable *gprintable, xmlDocPtr doc, xmlNodePtr node);
 
 static GtkTextBuffer *get_buffer(GnomeCanvasRichTextExt *text);
 static gint blink_cb(gpointer data);
@@ -159,6 +159,7 @@ static void
 gnome_canvas_rich_text_ext_print_init (GPrintableIface *iface)
 {
 	iface->print = gnome_canvas_rich_text_ext_print;
+	iface->export_svg = gnome_canvas_rich_text_ext_export_svg;
 }
 
 GType
@@ -692,6 +693,9 @@ gnome_canvas_rich_text_ext_unrealize(GnomeCanvasItem *item)
 {
 	GnomeCanvasRichTextExt *text = GNOME_CANVAS_RICH_TEXT_EXT(item);
 
+	/*remove idle calls to request_update */
+	while (	g_idle_remove_by_data (item));
+
 	gnome_canvas_rich_text_ext_destroy_layout(text);
 
 	(* GNOME_CANVAS_ITEM_CLASS(parent_class)->unrealize)(item);
@@ -1671,6 +1675,8 @@ gnome_canvas_rich_text_ext_stop_cursor_blink(GnomeCanvasRichTextExt *text)
 static gboolean
 request_update(gpointer data)
 {
+	if (!GNOME_IS_CANVAS_RICH_TEXT_EXT (data))
+		return;
 	GnomeCanvasRichTextExt *text = GNOME_CANVAS_RICH_TEXT_EXT(data);
 
 	gnome_canvas_item_request_update(GNOME_CANVAS_ITEM(text));
@@ -2276,28 +2282,17 @@ PangoLayout* gnome_canvas_rich_text_ext_get_pango_layout (GnomeCanvasRichTextExt
 static void 
 gnome_canvas_rich_text_ext_print       (GPrintable *gprintable, GnomePrintContext *pc)
 {
-	gint i, top, bottom, firsty;
+	gint top, bottom, firsty;
 	GSList *lines;
 	GtkTextLine *text_line;
-	PangoLayoutIter *iter;
-	PangoGlyphItem *item;
-	PangoRectangle logical_rect;
 	GtkTextLineDisplay *display;
 	GtkTextIter text_iter;
 	GdkRectangle rect;
 	GnomeCanvasRichTextExt *text;
-	GnomeFont *font;
-	GnomeFontFace *face;
-	GnomeGlyphList *glyph_list;
-	PangoFontDescription *desc;
-	GSList *extra_attrs_list;
-	gboolean foreground_set = FALSE;
-	guint16 foreground_red = 0, foreground_green = 0, foreground_blue = 0;
-	gboolean background_set = FALSE;
-	guint16 background_red = 0, background_green = 0, background_blue = 0;
 	g_return_if_fail (GNOME_IS_CANVAS_RICH_TEXT_EXT (gprintable));
 	text = GNOME_CANVAS_RICH_TEXT_EXT (gprintable);
-	g_return_if_fail (gtk_text_buffer_get_char_count (text->_priv->buffer) > 0);
+	if (gtk_text_buffer_get_char_count (text->_priv->buffer) == 0)
+		return;
 	gtk_text_layout_get_size (text->_priv->layout, NULL, &bottom);
 	top = 0;
 	if (bottom <= top) return;
@@ -2313,7 +2308,41 @@ gnome_canvas_rich_text_ext_print       (GPrintable *gprintable, GnomePrintContex
 		gnome_print_translate (pc, text->_priv->x, text->_priv->y + top);
 		gnome_print_pango_layout_print (pc, display->layout);
 		gnome_print_grestore (pc);
+		lines = lines->next;
+		firsty++;
+	}
+}
+
+extern void pango_layout_to_svg (PangoLayout* layout, xmlDocPtr doc, xmlNodePtr node, double x, double y);
+
+static void
+gnome_canvas_rich_text_ext_export_svg (GPrintable *printable, xmlDocPtr doc, xmlNodePtr node)
+{
+	gint top, bottom, firsty;
+	GSList *lines;
+	GtkTextLine *text_line;
+	GtkTextLineDisplay *display;
+	GtkTextIter text_iter;
+	GdkRectangle rect;
+	GnomeCanvasRichTextExt *text;
+	xmlNodePtr child;
+	char *buf;
+	g_return_if_fail (GNOME_IS_CANVAS_RICH_TEXT_EXT (printable));
+	text = GNOME_CANVAS_RICH_TEXT_EXT (printable);
+	if (gtk_text_buffer_get_char_count (text->_priv->buffer) == 0)
+		return;
+	gtk_text_layout_get_size (text->_priv->layout, NULL, &bottom);
+	top = 0;
+	if (bottom <= top) return;
+	lines = gtk_text_layout_get_lines (text->_priv->layout, top, bottom, &firsty);
+	firsty = 0;
+	while (lines) {
+		text_line = lines->data;
+		display = gtk_text_layout_get_line_display (text->_priv->layout, text_line, TRUE);
+		gtk_text_buffer_get_iter_at_line (text->_priv->buffer, &text_iter, firsty);
 		gnome_canvas_rich_text_ext_get_iter_location (text, &text_iter, &rect);
+		top = rect.y;
+		pango_layout_to_svg (display->layout, doc, node, text->_priv->x, text->_priv->y + top);
 		lines = lines->next;
 		firsty++;
 	}
