@@ -28,14 +28,30 @@
 #include "plugin.h"
 #include "lib/application.h"
 #include "arrowtool.h"
+#include "retrosynthesis.h"
+#include "retrosynthesisarrow.h"
+#include <glib/gi18n.h>
 	
 gcpArrowsPlugin plugin;
 
-gcpArrowsPlugin::gcpArrowsPlugin(): gcpPlugin()
+static Object* CreateRetrosynthesis ()
 {
+	return new gcpRetrosynthesis ();
 }
 
-gcpArrowsPlugin::~gcpArrowsPlugin()
+static Object* CreateRetrosynthesisArrow ()
+{
+	return new gcpRetrosynthesisArrow ();
+}
+
+gcpArrowsPlugin::gcpArrowsPlugin (): gcpPlugin ()
+{
+	RetrosynthesisType = Object::AddType ("retrosynthesis", CreateRetrosynthesis);
+	Object::SetCreationLabel (RetrosynthesisType, _("Create a new retrosynthesis pathway"));
+	RetrosynthesisArrowType = Object::AddType ("retrosynthesis-arrow", CreateRetrosynthesisArrow);
+}
+
+gcpArrowsPlugin::~gcpArrowsPlugin ()
 {
 }
 
@@ -45,4 +61,8 @@ void gcpArrowsPlugin::Populate (gcpApplication* App)
 	new gcpArrowTool (App);
 	new gcpArrowTool (App, gcpReversibleArrow);
 	new gcpArrowTool (App, gcpDoubleHeadedArrow);
+	new gcpArrowTool (App, gcpDoubleQueuedArrow);
+	Object::AddRule ("retrosynthesis", RuleMustContain, "molecule");
+	Object::AddRule ("retrosynthesis", RuleMustContain, "retrosynthesis-arrow");
+	Object::AddRule ("retrosynthesis-arrow", RuleMustBeIn, "retrosynthesis");
 }
