@@ -57,12 +57,10 @@ gcpSelectionTool::gcpSelectionTool(gcpApplication *App): gcpTool(App, "Select")
 	m_pApp->SetCallback("rotate", (GCallback) on_rotate);
 	m_pApp->SetCallback("merge", (GCallback) on_merge);
 	m_bRotate = false;
-	m_UIManager = gtk_ui_manager_new ();
 }
 
 gcpSelectionTool::~gcpSelectionTool()
 {
-	g_object_unref (m_UIManager);
 }
 
 bool gcpSelectionTool::OnClicked()
@@ -382,13 +380,10 @@ void gcpSelectionTool::CreateGroup ()
 	}
 }
 
-bool gcpSelectionTool::OnRightButtonClicked ()
+bool gcpSelectionTool::OnRightButtonClicked (GtkUIManager *UIManager)
 {
 	// first destroy the GtkUIManager
 	GtkWidget *w;
-	g_object_unref (m_UIManager);
-	m_UIManager = gtk_ui_manager_new ();
-	GtkActionGroup *group = gtk_action_group_new ("selection");
 	if (m_pData->SelectedObjects.size () > 1) {
 		set<TypeId> possible_types, types, wrong_types;
 		list<Object*>::iterator  i = m_pData->SelectedObjects.begin (),
@@ -409,17 +404,17 @@ bool gcpSelectionTool::OnRightButtonClicked ()
 			m_Type = *possible_types.begin();
 			const string &label = Object::GetCreationLabel (m_Type);
 			if (label.size ()) {
+				GtkActionGroup *group = gtk_action_group_new ("selection");
 				GtkAction *action = gtk_action_new ("create_group", label.c_str (), NULL, NULL);
 				gtk_action_group_add_action (group, action);
-				gtk_ui_manager_insert_action_group (m_UIManager, group, 0);
+				gtk_ui_manager_insert_action_group (UIManager, group, 0);
 				char buf[] = "<ui><popup><menuitem action='create_group'/></popup></ui>";
-				m_uiIds.push_front (gtk_ui_manager_add_ui_from_string (m_UIManager, buf, -1, NULL));
-				w = gtk_ui_manager_get_widget (m_UIManager, "/popup/create_group");
+				m_uiIds.push_front (gtk_ui_manager_add_ui_from_string (UIManager, buf, -1, NULL));
+				w = gtk_ui_manager_get_widget (UIManager, "/popup/create_group");
 				g_signal_connect_swapped (w, "activate", G_CALLBACK (on_create_group), this);
+				return true;
 			}
-			w = gtk_ui_manager_get_widget (m_UIManager, "/popup");
-			gtk_menu_popup (GTK_MENU (w), NULL, NULL, NULL, this, 3,  gtk_get_current_event_time ());
 		}
 	}
-	return true;
+	return false;
 }
