@@ -6,7 +6,7 @@
  *
  * Copyright (C) 2003-2004
  *
- * Developed by Jean Bréfort <jean.brefort@ac-dijon.fr>
+ * Developed by Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -42,6 +42,8 @@ gcpFragmentTool::gcpFragmentTool(gcpApplication *App): gcpTextTool(App, "Fragmen
 
 gcpFragmentTool::~gcpFragmentTool()
 {
+	if (ClipboardData)
+		xmlFree (ClipboardData);
 }
 
 static void on_get_data(GtkClipboard *clipboard, GtkSelectionData *selection_data,  guint info, gcpFragmentTool* tool)
@@ -81,6 +83,13 @@ bool gcpFragmentTool::OnClicked()
 				gcpBond *pBond = (gcpBond*)pAtom->GetFirstBond(i);
 				gcpFragment *pFragment = new gcpFragment(x, y);
 				gcpAtom* pFragAtom = (gcpAtom*)pFragment->m_Atom;
+				map<string, Object*>::iterator ie;
+				Object* electron = pAtom->GetFirstChild (ie);
+				while (electron){
+					m_pView->Remove (electron);
+					delete electron;
+					electron = pAtom->GetNextChild (ie);
+				}
 				pMol->Remove(pAtom);
 				pAtom->SetParent(NULL);
 				pMol->AddFragment(pFragment);
@@ -255,20 +264,20 @@ bool gcpFragmentTool::Unselect()
 	return false;
 }
 
-void gcpFragmentTool::OnGetData(GtkClipboard *clipboard, GtkSelectionData *selection_data,  guint info)
+void gcpFragmentTool::OnGetData (GtkClipboard *clipboard, GtkSelectionData *selection_data,  guint info)
 {
-	xmlDocPtr pDoc = gcpWidgetData::GetXmlDoc(clipboard);
-	if (ClipboardData)
-	{
-		xmlFree(ClipboardData);
+	xmlDocPtr pDoc = gcpWidgetData::GetXmlDoc (clipboard);
+	if (ClipboardData) {
+		xmlFree (ClipboardData);
+		ClipboardData = NULL;
 	} 
 	ClipboardDataType = info;
 	gint size;
 	if (info)
 	{
-		ClipboardData = xmlNodeGetContent(pDoc->children->children);
+		ClipboardData = xmlNodeGetContent (pDoc->children->children);
 		size = strlen((char*) ClipboardData);
-		gtk_selection_data_set_text(selection_data, (const gchar*) ClipboardData, size);
+		gtk_selection_data_set_text (selection_data, (const gchar*) ClipboardData, size);
 	}
 	else
 	{
