@@ -44,30 +44,21 @@ int main(int argc, char *argv[])
 	textdomain(GETTEXT_PACKAGE);
 
 	GnomeProgram* prog = gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv, 
-//                   GNOME_PARAM_POPT_TABLE, NULL,
                    GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
 	g_object_get(G_OBJECT(prog), GNOME_PARAM_POPT_CONTEXT, &pctx, NULL);
 
-/*	if (bonobo_server_flag)
-	{
-		BONOBO_FACTORY_INIT ("gchempaint", VERSION, &argc, argv);		
-		return bonobo_generic_factory_main ("OAFIID:gchempaint_factory", gchempaint_factory, NULL);
-	}
-	else*/
-	{
-		gcpStandaloneApp* App = new gcpStandaloneApp();
-		const char ** args = poptGetArgs(pctx);
-		int i = 0;
-		if (args)
-			while(args[i])
-			{
-				App->FileProcess(args[i++], 0, false, NULL);
-			}	
-		else App->OnFileNew();
+	gcpStandaloneApp* App = new gcpStandaloneApp();
+	const char ** args = poptGetArgs(pctx);
+	int i = 0;
+	if (args)
+		while(args[i])
+		{
+			App->FileProcess(args[i++], 0, false, NULL);
+		}
+	else App->OnFileNew();
 
-		gtk_main();
-		delete App;
-	}
+	gtk_main();
+	delete App;
 
 	return 0;
 }
